@@ -1,5 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-
 /*
  *  Copyright (C) 2003 Takuro Ashie
  *
@@ -20,20 +18,34 @@
 
 #include "kz-tablabel.h"
 
+#include <stdlib.h>
 #include <string.h>
 #include "gobject-utils.h"
+#include "gtk-utils.h"
 #include "intl.h"
 
-/* XPM */
-#include "pixmaps/small-close.xpm"
 
-static void     kz_tab_label_class_init         (KzTabLabelClass *class);
+struct _KzTabLabelPriv
+{
+	gint start_x, start_y;
+	gboolean moved;
+};
+
+
+static void     kz_tab_label_class_init         (KzTabLabelClass *klass);
 static void     kz_tab_label_init               (KzTabLabel *kztab);
-static void     kz_tab_label_destroy            (GtkObject *object);
+static void     kz_tab_label_dispose            (GObject *object);
+static void     kz_tab_label_finalize           (GObject *object);
 static void     kz_tab_label_realize            (GtkWidget *widget);
 static void     kz_tab_label_unrealize          (GtkWidget *widget);
-static void     kz_tab_label_close_cb           (GtkWidget *button,
-						 KzTabLabel *kztab);
+static gboolean kz_tab_label_button_press       (GtkWidget *widget,
+						 GdkEventButton *event);
+static gboolean kz_tab_label_motion_notify      (GtkWidget *widget,
+						 GdkEventMotion *event);
+static gboolean kz_tab_label_button_release     (GtkWidget *widget,
+						 GdkEventButton *event);
+static gboolean kz_tab_label_scroll_event       (GtkWidget *widget,
+						 GdkEventScroll *event);
 static gboolean kz_tab_label_drag_motion        (GtkWidget *widget,
 						 GdkDragContext *drag_context,
 						 gint x, gint y,
@@ -53,11 +65,14 @@ static void     kz_tab_label_drag_data_received (GtkWidget *widget,
 static void     kz_tab_label_move_page          (KzTabLabel *kztab,
 						 KzTabLabel *dest_kztab);
 
+static void     cb_close_button_clicked         (GtkWidget *button,
+						 KzTabLabel *kztab);
+
 /* callbacks for mozilla */
-static void title_changed_cb (GtkMozEmbed *embed, KzTabLabel *kztab);
-static void net_start_cb     (GtkMozEmbed *embed, KzTabLabel *kztab);
-static void net_stop_cb      (GtkMozEmbed *embed, KzTabLabel *kztab);
-static void destroy_cb       (GtkObject *object, KzTabLabel *kztab);
+static void cb_title_changed (GtkMozEmbed *embed, KzTabLabel *kztab);
+static void cb_net_start     (GtkMozEmbed *embed, KzTabLabel *kztab);
+static void cb_net_stop      (GtkMozEmbed *embed, KzTabLabel *kztab);
+static void cb_destroy       (GtkObject *object, KzTabLabel *kztab);
 
 
 static GtkEventBoxClass *parent_class = NULL;
@@ -85,25 +100,30 @@ KZ_OBJECT_GET_TYPE(kz_tab_label, "KzTabLabel", KzTabLabel,
 		   GTK_TYPE_EVENT_BOX)
 
 static void
-kz_tab_label_class_init (KzTabLabelClass *class)
+kz_tab_label_class_init (KzTabLabelClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *gobject_class;
 	GtkWidgetClass *widget_class;
 
-	parent_class = g_type_class_peek_parent (class);
+	parent_class = g_type_class_peek_parent (klass);
 
-	object_class = (GtkObjectClass *) class;
-	widget_class = (GtkWidgetClass *) class;
+	gobject_class = (GObjectClass *) klass;
+	widget_class  = (GtkWidgetClass *) klass;
 
 	/* GtkObject signals */
-	object_class->destroy = kz_tab_label_destroy;
+	gobject_class->dispose  = kz_tab_label_dispose;
+	gobject_class->finalize = kz_tab_label_finalize;
 
 	/* GtkWidget signales */
-	widget_class->realize            = kz_tab_label_realize;
-	widget_class->unrealize          = kz_tab_label_unrealize;
-	widget_class->drag_motion        = kz_tab_label_drag_motion;
-	widget_class->drag_data_get      = kz_tab_label_drag_data_get;
-	widget_class->drag_data_received = kz_tab_label_drag_data_received;
+	widget_class->realize              = kz_tab_label_realize;
+	widget_class->unrealize            = kz_tab_label_unrealize;
+	widget_class->button_press_event   = kz_tab_label_button_press;
+	widget_class->scroll_event         = kz_tab_label_scroll_event;
+	widget_class->motion_notify_event  = kz_tab_label_motion_notify;
+	widget_class->button_release_event = kz_tab_label_button_release;
+	widget_class->drag_motion          = kz_tab_label_drag_motion;
+	widget_class->drag_data_get        = kz_tab_label_drag_data_get;
+	widget_class->drag_data_received   = kz_tab_label_drag_data_received;
 }
 
 static void
@@ -117,12 +137,13 @@ kz_tab_label_init (KzTabLabel *kztab)
 	kztab->hbox         = NULL;
 	kztab->label        = NULL;
 	kztab->close_button = NULL;
+	kztab->tooltips     = gtk_tooltips_new();
 	for (i = 0; i < KZ_TAB_LABEL_N_STATE; i++)
 		kztab->style[i] = NULL;
 }
 
 static void
-kz_tab_label_destroy (GtkObject *object)
+kz_tab_label_dispose (GObject *object)
 {
 	KzTabLabel *kztab;
 
@@ -133,34 +154,39 @@ kz_tab_label_destroy (GtkObject *object)
 	if (kztab->kzembed)
 	{
 		g_signal_handlers_disconnect_by_func(G_OBJECT(kztab->kzembed),
-						     net_start_cb, kztab);
+						     cb_net_start, kztab);
 		g_signal_handlers_disconnect_by_func(G_OBJECT(kztab->kzembed),
-						     net_stop_cb, kztab);
+						     cb_net_stop, kztab);
 		g_signal_handlers_disconnect_by_func(G_OBJECT(kztab->kzembed),
-						     title_changed_cb, kztab);
+						     cb_title_changed, kztab);
 		g_signal_handlers_disconnect_by_func(G_OBJECT(kztab->kzembed),
-						     destroy_cb, kztab);
+						     cb_destroy, kztab);
 	}
 	kztab->kzembed = NULL;
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		GTK_OBJECT_CLASS (parent_class)->destroy(object);
+	if (kztab->tooltips)
+	{
+		g_object_unref(kztab->tooltips);
+	}
+	kztab->tooltips = NULL;
+
+	if (G_OBJECT_CLASS (parent_class)->dispose)
+		G_OBJECT_CLASS (parent_class)->dispose(object);
 }
 
+KZ_OBJECT_FINALIZE (kz_tab_label, KzTabLabel)
+
 GtkWidget *
 kz_tab_label_new (KzWindow *kz, KzMozEmbed *kzembed)
 {
 	KzTabLabel *kztab = KZ_TAB_LABEL(g_object_new(KZ_TYPE_TAB_LABEL, NULL));
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
 	GtkWidget *image;
-	const gchar *title;
+	gchar *title;
 
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
 	g_return_val_if_fail(KZ_IS_MOZ_EMBED(kzembed), NULL);
 
-	title = kz_moz_embed_get_title(kzembed);
-	if (!title || !*title) title = _("no title");
+	title = kz_moz_embed_ensure_title(kzembed);
 
 	kztab->kz           = kz;
 	kztab->kzembed      = kzembed;
@@ -168,6 +194,11 @@ kz_tab_label_new (KzWindow *kz, KzMozEmbed *kzembed)
 	kztab->label        = gtk_label_new(title);
 	kztab->close_button = gtk_button_new();
 
+	kztab->priv = g_new0(KzTabLabelPriv, 1);
+	kztab->priv->start_x = 0;
+	kztab->priv->start_y = 0;
+	kztab->priv->moved   = FALSE;
+
 	/*  hbox */
 	gtk_container_add (GTK_CONTAINER(kztab), kztab->hbox);
 	gtk_widget_show(kztab->hbox);
@@ -180,20 +211,13 @@ kz_tab_label_new (KzWindow *kz, KzMozEmbed *kzembed)
 	gtk_widget_set_size_request(kztab->close_button, 14, 14);
 	gtk_button_set_relief(GTK_BUTTON(kztab->close_button), GTK_RELIEF_NONE);
 	g_signal_connect(G_OBJECT(kztab->close_button), "clicked", 
-			 G_CALLBACK(kz_tab_label_close_cb), kztab);
+			 G_CALLBACK(cb_close_button_clicked), kztab);
 
 	gtk_box_pack_start(GTK_BOX(kztab->hbox), kztab->label, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(kztab->hbox), kztab->close_button, FALSE, FALSE, 0);
 
-	pixmap = gdk_pixmap_colormap_create_from_xpm_d
-			(GTK_WIDGET(kz)->window,
-			 gtk_widget_get_colormap(GTK_WIDGET(kztab)),
-			 &mask,
-			 NULL,
-			 small_close);
-	image = gtk_image_new_from_pixmap(pixmap,mask);
-	gdk_pixmap_unref(pixmap);
-	gdk_bitmap_unref(mask);
+	image = gtk_image_new_from_stock(GTK_STOCK_CLOSE,
+					 GTK_ICON_SIZE_MENU);
 	gtk_container_add(GTK_CONTAINER(kztab->close_button), image);
 
 	gtk_widget_show(image);
@@ -210,15 +234,21 @@ kz_tab_label_new (KzWindow *kz, KzMozEmbed *kzembed)
                           url_drag_types,n_url_drag_types,
 			  GDK_ACTION_MOVE);
 
+	/* tooltips */
+	gtk_tooltips_set_tip(kztab->tooltips, GTK_WIDGET(kztab),
+			     title, NULL);
+
 	/* set signals */
 	g_signal_connect(G_OBJECT(kzembed), "title",
-			 G_CALLBACK(title_changed_cb), kztab);
+			 G_CALLBACK(cb_title_changed), kztab);
 	g_signal_connect(G_OBJECT(kzembed), "net-start",
-			 G_CALLBACK(net_start_cb), kztab);
+			 G_CALLBACK(cb_net_start), kztab);
 	g_signal_connect(G_OBJECT(kzembed), "net-stop",
-			 G_CALLBACK(net_stop_cb), kztab);
+			 G_CALLBACK(cb_net_stop), kztab);
 	g_signal_connect(G_OBJECT(kzembed), "destroy",
-			 G_CALLBACK(destroy_cb), kztab);
+			 G_CALLBACK(cb_destroy), kztab);
+
+	g_free(title);
 
 	return GTK_WIDGET(kztab);
 }
@@ -241,7 +271,12 @@ void
 kz_tab_label_set_text(KzTabLabel *kztab, const gchar *text)
 {
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
+
+	if (!text || !*text) text = _("No title");
+
 	gtk_label_set_text(GTK_LABEL(kztab->label), text);
+	gtk_tooltips_set_tip(kztab->tooltips, GTK_WIDGET(kztab),
+			     text, NULL);
 }
 
 void
@@ -325,52 +360,99 @@ kz_tab_label_unrealize (GtkWidget *widget)
 	}
 
 	if (GTK_WIDGET_CLASS (parent_class)->unrealize)
-		GTK_WIDGET_CLASS (parent_class)->unrealize(widget);
+		return GTK_WIDGET_CLASS (parent_class)->unrealize(widget);
 }
 
-/* callbacks for mozilla */
-
-static void
-title_changed_cb (GtkMozEmbed *embed, KzTabLabel *kztab)
+static gboolean
+kz_tab_label_button_press (GtkWidget *widget, GdkEventButton *event)
 {
-	const gchar *title;
+	KzTabLabel *kztab = KZ_TAB_LABEL(widget);
 
-	g_return_if_fail(KZ_IS_MOZ_EMBED(embed));
-	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
+	kztab->priv->start_x = event->x;
+	kztab->priv->start_y = event->y;
+	kztab->priv->moved = FALSE;
 
-	title = kz_moz_embed_get_title(kztab->kzembed);
-	if (!title || !*title) title = _("no title");
-
-	gtk_label_set_text(GTK_LABEL(kztab->label), (gchar *) title);
+	if (GTK_WIDGET_CLASS(parent_class)->button_press_event)
+		GTK_WIDGET_CLASS(parent_class)->button_press_event(widget, event);
+	return FALSE;
 }
 
-static void
-net_start_cb (GtkMozEmbed *embed, KzTabLabel *kztab)
+static gboolean
+kz_tab_label_motion_notify (GtkWidget *widget, GdkEventMotion *event)
 {
-	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
-	kz_tab_label_set_state(kztab, KZ_TAB_LABEL_STATE_LOADING);
+	KzTabLabel *kztab = KZ_TAB_LABEL(widget);
+
+	if (abs(event->x - kztab->priv->start_x) > 2 ||
+	    abs(event->y - kztab->priv->start_y) > 2)
+	{
+		kztab->priv->moved = TRUE;
+	}
+
+	if (GTK_WIDGET_CLASS(parent_class)->motion_notify_event)
+		return GTK_WIDGET_CLASS(parent_class)->motion_notify_event(widget, event);
+	return FALSE;
 }
 
-static void
-net_stop_cb (GtkMozEmbed *embed, KzTabLabel *kztab)
+static gboolean
+kz_tab_label_button_release (GtkWidget *widget, GdkEventButton *event)
 {
-	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
-	kz_tab_label_set_state(kztab, KZ_TAB_LABEL_STATE_NORMAL);
+	KzTabLabel *kztab = KZ_TAB_LABEL(widget);
+
+	if (event->button == 2 && !kztab->priv->moved)
+	{
+		/* Load URL in clipboard */
+		/* return TRUE; */
+	}
+	else if (event->button == 3 && !kztab->priv->moved)
+	{
+		kz_tab_label_popup_menu_modal(kztab,
+					      event->button,
+					      event->time);
+		return TRUE;
+	}
+
+	kztab->priv->start_x = 0;
+	kztab->priv->start_y = 0;
+	kztab->priv->moved = FALSE;
+
+	if (GTK_WIDGET_CLASS(parent_class)->button_release_event)
+		return GTK_WIDGET_CLASS(parent_class)->button_release_event(widget, event);
+	return FALSE;
 }
 
-static void
-kz_tab_label_close_cb (GtkWidget *button, KzTabLabel *kztab)
+
+static gboolean
+kz_tab_label_scroll_event (GtkWidget *widget, GdkEventScroll *event)
 {
-	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
-	kz_window_close_tab(kztab->kz, GTK_WIDGET(kztab->kzembed));
+	KzTabLabel *kztab;
+	GtkNotebook *notebook;
+	gboolean retval = FALSE;
+
+	kztab = KZ_TAB_LABEL(widget);
+	notebook = GTK_NOTEBOOK(kztab->kz->notebook);
+
+	switch (event->direction) {
+	case GDK_SCROLL_UP:
+	case GDK_SCROLL_LEFT:
+		gtk_notebook_prev_page(notebook);
+		retval = TRUE;
+		break;
+	case GDK_SCROLL_DOWN:
+	case GDK_SCROLL_RIGHT:
+		gtk_notebook_next_page(notebook);
+		retval = TRUE;
+		break;
+	default:
+		g_warning ("Invalid scroll direction!");
+		break;
+	}
+
+	if (GTK_WIDGET_CLASS(parent_class)->scroll_event)
+		return GTK_WIDGET_CLASS(parent_class)->scroll_event(widget, event)
+			|| retval;
+	return retval;
 }
 
-static void
-destroy_cb (GtkObject *object, KzTabLabel *kztab)
-{
-	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
-	kztab->kzembed = NULL;
-}
 
 static gboolean
 kz_tab_label_drag_motion (GtkWidget *widget,
@@ -555,4 +637,223 @@ kz_tab_label_move_page(KzTabLabel *kztab, KzTabLabel *dest_kztab)
 	}
 
 	gtk_notebook_reorder_child (notebook, src_page, dest_pos);
+	kz_window_update_nav_buttons
+		(kztab->kz, KZ_MOZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kztab->kz)));
+}
+
+static void
+cb_close_button_clicked (GtkWidget *button, KzTabLabel *kztab)
+{
+	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
+	kz_window_close_tab(kztab->kz, GTK_WIDGET(kztab->kzembed));
+}
+
+/* callbacks for mozilla */
+
+static void
+cb_title_changed (GtkMozEmbed *embed, KzTabLabel *kztab)
+{
+	gchar *title;
+
+	g_return_if_fail(KZ_IS_MOZ_EMBED(embed));
+	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
+
+	title = kz_moz_embed_ensure_title(kztab->kzembed);
+	kz_tab_label_set_text(kztab, title);
+	g_free(title);
+}
+
+static void
+cb_net_start (GtkMozEmbed *embed, KzTabLabel *kztab)
+{
+	gchar *title;
+
+	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
+	kz_tab_label_set_state(kztab, KZ_TAB_LABEL_STATE_LOADING);
+
+	title = kz_moz_embed_ensure_title(kztab->kzembed);
+	kz_tab_label_set_text(kztab, title);
+	g_free(title);
+}
+
+static void
+cb_net_stop (GtkMozEmbed *embed, KzTabLabel *kztab)
+{
+	gchar *title;
+
+	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
+	kz_tab_label_set_state(kztab, KZ_TAB_LABEL_STATE_NORMAL);
+
+	title = kz_moz_embed_ensure_title(kztab->kzembed);
+	kz_tab_label_set_text(kztab, title);
+	g_free(title);
+}
+
+static void
+cb_destroy (GtkObject *object, KzTabLabel *kztab)
+{
+	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
+	kztab->kzembed = NULL;
+}
+
+
+
+/*****************************************************************************
+ *                                                                           *
+ *                               Popup menu                                  *
+ *                                                                           *
+ *****************************************************************************/
+static void
+cb_reload (KzTabLabel *kztab, guint action, gpointer data)
+{
+	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
+
+	gtk_moz_embed_reload(GTK_MOZ_EMBED(kztab->kzembed),
+			     GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
+}
+
+static void
+cb_stop (KzTabLabel *kztab, guint action, gpointer data)
+{
+	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
+
+	gtk_moz_embed_stop_load(GTK_MOZ_EMBED(kztab->kzembed));
+}
+
+static void
+cb_close (KzTabLabel *kztab, guint action, gpointer data)
+{
+	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
+
+	kz_window_close_tab(kztab->kz, GTK_WIDGET(kztab->kzembed));
+}
+
+static void
+cb_close_all (KzTabLabel *kztab, guint action, gpointer data)
+{
+	GtkNotebook *notebook;
+	gint num, pos;
+	gint i;
+
+	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
+	notebook = GTK_NOTEBOOK(kztab->kz->notebook);
+
+	num = gtkutil_notebook_get_n_pages(notebook);
+	pos = gtk_notebook_page_num(notebook, GTK_WIDGET(kztab->kzembed));
+
+	if (num < 2 || pos < 0) return;
+
+	for (i = num - 1; i >= 0; i--)
+	{
+		GtkWidget *widget = KZ_WINDOW_NTH_PAGE(kztab->kz, i);
+
+		if (i == pos) continue;
+		if (i > pos && action == 0) continue;
+		if (i < pos && action == 1) continue;
+
+		kz_window_close_tab_idle(kztab->kz, widget);
+	}
+}
+
+static void
+cb_copy (KzTabLabel *kztab, guint action, gpointer data)
+{
+	const gchar *text = NULL;
+
+	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
+
+	switch (action) {
+	case 0:
+		text = kz_moz_embed_get_title(KZ_MOZ_EMBED(kztab->kzembed));
+		break;
+	case 1:
+		text = kz_moz_embed_get_location(KZ_MOZ_EMBED(kztab->kzembed));
+		break;
+	default:
+		return;
+		break;
+	}
+
+	if (text && *text)
+	{
+		GtkClipboard *clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+		gtk_clipboard_set_text(clipboard, text, strlen(text));
+		clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+		gtk_clipboard_set_text(clipboard, text, strlen(text));
+	}
+}
+
+static void
+cb_popup_menu_hide ()
+{
+	gtk_main_quit();
+}
+
+static GtkItemFactoryEntry popup_menu[] =
+{
+	{N_("/Reloa_d"),      NULL, G_CALLBACK(cb_reload),  0, NULL, NULL},
+	{N_("/_Stop"),        NULL, G_CALLBACK(cb_stop),    0, NULL, NULL},
+
+	{N_("/---"),          NULL, NULL, 0, "<Separator>", NULL},
+
+	{N_("/_Close"),                  NULL, G_CALLBACK(cb_close),     0, NULL, NULL},
+	{N_("/Close all _left tab"),     NULL, G_CALLBACK(cb_close_all), 0, NULL, NULL},
+	{N_("/Close all _right tab"),    NULL, G_CALLBACK(cb_close_all), 1, NULL, NULL},
+	{N_("/Close with_out this tab"), NULL, G_CALLBACK(cb_close_all), 2, NULL, NULL},
+
+	{N_("/---"),          NULL, NULL, 0, "<Separator>", NULL},
+
+	{N_("/Copy _Title"),  NULL, G_CALLBACK(cb_copy),  0, NULL, NULL},
+	{N_("/Copy _URL"),    NULL, G_CALLBACK(cb_copy),  1, NULL, NULL},
+};
+
+static void
+set_popup_menu_sensitive (KzTabLabel *kztab, GtkWidget *menu)
+{
+   GtkWidget *item;
+   GtkItemFactory *ifactory;
+   gint pos, num;
+
+   pos = gtk_notebook_page_num(GTK_NOTEBOOK(kztab->kz->notebook),
+			       GTK_WIDGET(kztab->kzembed));
+   num = gtkutil_notebook_get_n_pages(GTK_NOTEBOOK(kztab->kz->notebook));
+
+   ifactory = gtk_item_factory_from_widget(menu);
+
+   item = gtk_item_factory_get_item(ifactory, "/Stop");
+   gtk_widget_set_sensitive(item, kz_moz_embed_is_loading(kztab->kzembed));
+   item = gtk_item_factory_get_item(ifactory, "/Close all left tab");
+   gtk_widget_set_sensitive(item, !(pos == 0));
+   item = gtk_item_factory_get_item(ifactory, "/Close all right tab");
+   gtk_widget_set_sensitive(item, !(pos == num -1));
+   item = gtk_item_factory_get_item(ifactory, "/Close without this tab");
+   gtk_widget_set_sensitive(item, num > 1);
+}
+
+void
+kz_tab_label_popup_menu_modal (KzTabLabel *kztab, guint button, guint time)
+{
+	GtkItemFactory *factory = NULL;
+	GtkWidget *widget;
+
+	factory = gtk_item_factory_new (GTK_TYPE_MENU,
+					"<PopupMenu>", NULL);
+#ifdef ENABLE_NLS
+	gtk_item_factory_set_translate_func (factory,
+					     (GtkTranslateFunc) gettext,
+					     NULL, NULL);
+#endif /* ENABLE_NLS */
+	gtk_item_factory_create_items (factory,
+				       G_N_ELEMENTS(popup_menu),
+				       popup_menu,
+				       kztab);
+	widget = gtk_item_factory_get_widget (factory, "<PopupMenu>");
+	set_popup_menu_sensitive(kztab, widget);
+	g_signal_connect(G_OBJECT(widget), "hide",
+			 G_CALLBACK(cb_popup_menu_hide), kztab);
+	gtk_menu_popup(GTK_MENU(widget), NULL, NULL,
+		       NULL, NULL, button, time);
+	gtk_main();
+	gtk_widget_destroy(widget);
+	g_object_unref(factory);
 }
