@@ -32,8 +32,19 @@ G_BEGIN_DECLS
 #  define gtkutil_notebook_get_n_pages gtk_notebook_get_n_pages
 #endif /* (GTK_MAJOR_VERSION == 2) && (GTK_MAJOR_VERSION < 1) */
 
-void menu_position_under_widget (GtkMenu *menu, gint *x, gint *y,
-				 gboolean *push_in, gpointer user_data);
+void   gtkutil_menu_position_under_widget	(GtkMenu *menu,
+						 gint *x, gint *y,
+						 gboolean *push_in,
+						 gpointer user_data);
+
+GList *gtkutil_tree_selection_get_selected_rows	(GtkTreeSelection *selection,
+						 GtkTreeModel **model);
+GList *gtkutil_tree_selection_get_selected_references
+						(GtkTreeSelection *selection,
+						 GtkTreeModel **tree_model);
+void   gtkutil_list_store_swap			(GtkListStore *store,
+						 GtkTreeIter  *a,
+						 GtkTreeIter  *b);
 
 G_END_DECLS
 
